@@ -9,7 +9,7 @@ import os
 from spython.logger import bot
 
 
-def create(self, image_path, size=1024, sudo=False):
+def create(self, image_path, size=1024, sudo=False, singularity_options=None):
     """create will create a a new image
 
         Parameters
@@ -17,13 +17,13 @@ def create(self, image_path, size=1024, sudo=False):
         image_path: full path to image
         size: image sizein MiB, default is 1024MiB
         filesystem: supported file systems ext3/ext4 (ext[2/3]: default ext3
-
+        singularity_options: a list of options to provide to the singularity client
     """
     from spython.utils import check_install
 
     check_install()
 
-    cmd = self.init_command("image.create")
+    cmd = self.init_command("image.create", singularity_options)
     cmd = cmd + ["--size", str(size), image_path]
 
     output = self.run_command(cmd, sudo=sudo)
