@@ -9,7 +9,14 @@ from spython.logger import bot
 
 
 def start(
-    self, image=None, name=None, args=None, sudo=False, options=None, capture=False
+    self,
+    image=None,
+    name=None,
+    args=None,
+    sudo=False,
+    options=None,
+    capture=False,
+    singularity_options=None,
 ):
     """start an instance. This is done by default when an instance is created.
 
@@ -20,6 +27,7 @@ def start(
        sudo: if the user wants to run the command with sudo
        capture: capture output, default is False. With True likely to hang.
        args: arguments to provide to the instance (supported Singularity 3.1+)
+       singularity_options: a list of options to provide to the singularity client
        options: a list of tuples, each an option to give to the start command
                 [("--bind", "/tmp"),...]
 
@@ -49,7 +57,7 @@ def start(
     if "version 3" in self.version():
         subgroup = ["instance", "start"]
 
-    cmd = self._init_command(subgroup)
+    cmd = self._init_command(subgroup, singularity_options)
 
     # Add options, if they are provided
     if not isinstance(options, list):
